package main

import (
	"bufio"
	"flag"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"path"

	"github.com/jbarham/cdb"
)

func exitOnErr(err error) {
	if err != nil {
		log.Fatal(err)
	}
}

func usage() {
	fmt.Fprint(os.Stderr, "usage: cdb [-dm] [-k key [-s n]] db\n")
	os.Exit(2)
}

func dumpdb(out *bufio.Writer, in *bufio.Reader) {
	err := cdb.Dump(out, in)
	out.Flush()
	if err != nil {
		os.Exit(111)
	}
}


func makedb(db string, tmp *os.File) {
	tmpname := tmp.Name()

	exitOnErr(cdb.Make(tmp, bufio.NewReader(os.Stdin)))
	exitOnErr(tmp.Sync())
	exitOnErr(tmp.Close())
	exitOnErr(os.Rename(tmpname, db))
}

func get(db string, key string, skip int64) {
	c, err := cdb.Open(db)
	if err != nil {
		log.Fatal(err)
	}
	defer c.Close()
	data, err:= c.Data([]byte(key))
	if err != nil {
		log.Fatal(err)
	}
	fmt.Printf("%s\n", string(data))
}

func main() {
	var tmp *os.File
	var err error

	var key string
	var db string
	var skip int64
	var dump bool
	var make bool

	flag.StringVar(&key, "k", "", "Print record with key k")
	flag.Int64Var(&skip, "s", 0, "Skip the first s matching records")
        flag.BoolVar(&dump, "d", false, "Print records suitable for cbdmake")
        flag.BoolVar(&make, "m", false, "Writes a constant database to f")
	flag.Parse()

	args := flag.Args()

	if (len(args) != 1) {
		usage()
	}

	db = args[0]

	if (make) {
		dir, _ := path.Split(db)
		if (dir == "") {
			dir = "."
		}
		tmp, err = ioutil.TempFile(dir, ".*.cdb" )
		exitOnErr(err)
		makedb(db, tmp)
		os.Exit(0)
	} else if (dump) {
		in, out := bufio.NewReader(os.Stdin), bufio.NewWriter(os.Stdout)
		f, err := os.OpenFile(db, os.O_RDWR|os.O_CREATE, 0644)
		if err != nil {
			log.Fatal(err)
		}
		in = bufio.NewReader(f)
		dumpdb(out, in)
		os.Exit(0)
	}

	get(db, key, skip)
}
